from flask import Flask, render_template, Response, make_response
import threading
import socket
import qrcode
from zeroconf import ServiceInfo, Zeroconf, NonUniqueNameException
from screenx10sion.streamYvideo import generate_video_stream

app = Flask(__name__)

@app.route('/')
def index():
    try:
        return render_template('index.html')
    except Exception as e:
        print(f"[!] Error loading template: {e}")
        # Fallback HTML content if template is missing
        fallback_html = """
        <html>
        <head><title>ScreenX10sion</title></head>
        <body>
            <h1>ScreenX10sion</h1>
            <p>This is the fallback page. Template not found.</p>
            <img src="/video_feed" style="max-width: 100%;">
        </body>
        </html>
        """
        return make_response(fallback_html, 200)

@app.route('/video_feed')
def video_feed():
    return Response(generate_video_stream(),
                    mimetype='multipart/x-mixed-replace; boundary=frame')

def get_local_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('10.255.255.255', 1))  
        IP = s.getsockname()[0]
    except Exception:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP

def register_mdns_service(ip):
    desc = {'path': '/'}
    hostname = socket.gethostname()
    zeroconf = Zeroconf()

    info = ServiceInfo(
        "_http._tcp.local.",
        "screenx10sion._http._tcp.local.",
        addresses=[socket.inet_aton(ip)],
        port=5000,
        properties=desc,
        server=f"{hostname}.local.",
    )

    try:
        zeroconf.register_service(info, allow_name_change=True)
        print(f"[+] mDNS service registered as http://{info.name}")
    except NonUniqueNameException:
        print("[!] Service name conflict. Tried fallback, but failed.")
    except Exception as e:
        print(f"[!] mDNS registration error: {e}")

def show_qr():
    from PIL import Image, ImageDraw, ImageFont

    ip = get_local_ip()
    url = f"http://{ip}:5000"
    print(f"[+] Scan this QR to open second screen: {url}")

    qr = qrcode.make(url).convert("RGB")
    qr_width, qr_height = qr.size

    try:
        font = ImageFont.truetype("arial.ttf", 20)
    except:
        font = ImageFont.load_default()

    text1 = "Scan the QR to view the screen or go to the URL"
    text2 = url

    def get_text_size(text):
        bbox = font.getbbox(text)
        return bbox[2] - bbox[0], bbox[3] - bbox[1]

    text1_width, text1_height = get_text_size(text1)
    text2_width, text2_height = get_text_size(text2)

    max_text_width = max(text1_width, text2_width)
    padding = 20
    total_width = max(qr_width, max_text_width + 2 * padding)
    total_height = qr_height + text1_height + text2_height + 3 * padding

    new_img = Image.new("RGB", (total_width, total_height), "white")
    draw = ImageDraw.Draw(new_img)

    draw.text(((total_width - text1_width) // 2, padding), text1, font=font, fill="black")
    draw.text(((total_width - text2_width) // 2, padding + text1_height + 5), text2, font=font, fill="black")

    qr_x = (total_width - qr_width) // 2
    qr_y = padding + text1_height + text2_height + 10
    new_img.paste(qr, (qr_x, qr_y))

    new_img.show()

def run_flask():
    app.run(host='0.0.0.0', port=5000, debug=False, threaded=True)

def main():
    ip = get_local_ip()
    register_mdns_service(ip)
    threading.Thread(target=run_flask).start()
    show_qr()

if __name__ == '__main__':
    main()
